package de.opalpiraten.scenepainter.gui.render;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import de.opalpiraten.scenepainter.sys.*;

/**
 * A SceneElement is an entity, which is shown in the Scene, but unlike
 * a static image still is muteable.
 * <p>
 * This is the abstract superclass of all sceneelements. A concrete sublcass
 * must adhere to the following contract:<br>
 * Whenever an attribute, having an effect on the visual representation 
 * changes, the sceneelement must set SceneElement.cache to null (thus
 * invalidating the visual). At any time it must also be able to return 
 * a visual representation of itself via buildCache(). This visual 
 * representation may also be an error image in case the real thing cannot be
 * assembled.
 */
public abstract class SceneElement {

  /**
   * SceneElements may have names for identifying purposes
   */
  protected String name="???";
  
  /**
   * The location of this SceneElement in the Scene (top right corner).
   */
  protected Point position = new Point(0,0);
  
  /**
   * Set the alpha level of the SceneElement. Alpha ranges from 0.0f to 
   * 1.0f, the later being fully opaque
   */
  protected float alpha=1.0f;
  
  /**
   * Reference to the Project, the SceneElement should use for loading
   * resources.
   */
  protected Project project;
  
  /**
   * A cached version of the sceneelement. Subclasses must invalidate the 
   * cache (by setting it to null) each time they change in a way, that
   * affects the visual representation.
   */
  protected BufferedImage cache = null;
  
  
  /**
   * Set the Project, that is responsible for loading media resources
   */
  public void setProject(Project p) {
    project=p;
    cache=null;
  }
  
  /**
   * Set the name of this Element
   */
  public void setName(String n) {
    name=n;
    cache=null;
  }
  
  /**
   * Query the name of this Element
   */
  public String getName() {
    return name;
  }
  
  /**
   * Set Transparency 
   * @param a between 0.0f and 1.0f (the later being fully opaque
   */
  public void setAlpha(float a) {
    alpha=a;
    cache=null;
  }
  
  /**
   * Query Transparency level
   */
  public float getAlpha() {
    return alpha;
  }
  
  /**
   * Called by the Scene if user scrolls the mousewheel holding down
   * a modifier keys. SceneElements may override this method to quick
   * update their view (implementation is up to their choice).
   */
  public void scrollView(boolean forward) {}
    
  /**
   * Query the bounds of this SceneElement. 
   * @return the bounding rectangle
   */
  public Rectangle getBounds() {
    if (cache==null) cache=buildCache();
    return new Rectangle(position.x,position.y,cache.getWidth(),cache.getHeight()); 
  }
  
  /**
   * Specify the location where to paint this SceneElement onto the scene
   * @param p location of the top right corner.
   */
  public void setPosition(Point p) {
    position=p;
  }
  
  /**
   * Query the position of this element in the Scene
   * @return the position of the top right corner
   */
  public Point getPosition() {
    return position;
  }
  
  
  public String toString() {
    StringBuffer sb = new StringBuffer(name);
    
    Rectangle b = getBounds();
    sb.append(" (");
    sb.append(b.width);
    sb.append(" x ");
    sb.append(b.height);
    sb.append(" ");
    if (position.x>0) sb.append("+");
    sb.append(position.x);
    sb.append(" ");
    if (position.y>0) sb.append("+");
    sb.append(position.y);
    sb.append(")");
    
    return sb.toString();
  }
  
  /**
   * Render the cached SceneElement upon a graphics context.
   * @param g the Graphics object to render upon
   */
  public void paint(Graphics2D g) {
    if (cache==null) cache=buildCache();
    g.drawImage(cache,position.x,position.y,null);
  }
  
  
  /**
   * Will be called by paint(Graphics2D) and getBounds() whenever the cache is
   * null (invalid).
   * @return a cached and fully prepared visual of the sceneelement
   */
  public abstract BufferedImage buildCache();
  
}
